<?php

namespace tmc\revisionmanager\src\AdminPages\PageSettings;

/**
 * @author jakubkuranda@gmail.com
 * Date: 08.03.2018
 * Time: 09:39
 */

use shellpress\v1_2_0\src\Shared\AdminPageFramework\AdminPageTab;
use shellpress\v1_2_0\src\Shared\Front\Models\HtmlElement;
use tmc\revisionmanager\src\App;
use tmc\revisionmanager\src\Components\Revisions;
use WP_Post_Type;
use WP_Role;

class TabGeneral extends AdminPageTab {

    /**
     * Declaration of current element.
     */
    public function setUp() {

        //  ----------------------------------------
        //  Definition
        //  ----------------------------------------

        $this->pageFactory->addInPageTab(
            array(
                'page_slug'     =>  $this->pageSlug,
                'tab_slug'      =>  $this->tabSlug,
                'title'         =>  __( 'General settings', 'rm_tmc' ),
            )
        );

    }

    /**
     * Called while current component is loaded.
     */
    public function load() {

        //  ----------------------------------------
        //  Modifications
        //  ----------------------------------------

        add_filter( 'fields_' . $this->pageFactoryClassName,                array( $this, '_f_redefineField_postTypes_chosen' ) );
        add_filter( 'fields_' . $this->pageFactoryClassName,                array( $this, '_f_redefineField_capabilities_excludedEmails' ) );
        add_filter( 'fields_' . $this->pageFactoryClassName,                array( $this, '_f_redefineField_notifications_content' ) );

        //  ----------------------------------------
        //  Sections
        //  ----------------------------------------

        $this->pageFactory->addSettingSections(
            array(
                'section_id'            =>  'capabilities',
                'page_slug'             =>  $this->pageSlug,
                'tab_slug'              =>  $this->tabSlug,
                'title'                 =>  __( 'Capabilities', 'rm_tmc' ),
                'class'                 =>  'section-grid'
            ),
            array(
                'section_id'            =>  'postTypes',
                'page_slug'             =>  $this->pageSlug,
                'tab_slug'              =>  $this->tabSlug,
                'title'                 =>  __( 'Post types', 'rm_tmc' ),
                'class'                 =>  'section-grid'
            ),
            array(
                'section_id'            =>  'notifications',
                'page_slug'             =>  $this->pageSlug,
                'tab_slug'              =>  $this->tabSlug,
                'title'                 =>  __( 'Notifications', 'rm_tmc' ),
                'class'                 =>  'section-grid'
            ),
            array(
                'section_id'            =>  'submit',
                'page_slug'             =>  $this->pageSlug,
                'tab_slug'              =>  $this->tabSlug,
                'title'                 =>  null,
                'class'                 =>  'section-grid',
                'save'                  =>  false
            )
        );

        //  ----------------------------------------
        //  Fields
        //  ----------------------------------------

        $this->pageFactory->addSettingFields(
            'capabilities', //  section_id
            array(
                'field_id'              =>  'capCopy',
                'type'                  =>  'select',
                'title'                 =>  __( 'Copy creation capability', 'rm_tmc' ),
                'label'                 =>  $this->getAllCapabilitiesNames()
            ),
            array(
                'field_id'              =>  'capAccept',
                'type'                  =>  'select',
                'title'                 =>  __( 'Acceptation capability', 'rm_tmc' ),
                'label'                 =>  $this->getAllCapabilitiesNames()
            ),
            array(
                'field_id'              =>  'roleNotification',
                'type'                  =>  'select',
                'title'                 =>  __( 'Role for notification', 'rm_tmc' ),
                'label'                 =>  $this->getAllRolesNames(),
                'attributes'            =>  array()
            ),
            array(
                'field_id'              =>  'excludedEmails',
                'type'                  =>  'textarea',
                'title'                 =>  __( 'Excluded e-mails', 'rm_tmc' ),
                'attributes'            =>  array(),
                'tip'                   =>  __( 'These addresses will never receive any notification.', 'rm_tmc' ),
                'description'           =>  __( 'Comma separated e-mail addresses. Example: mail@host1.com, mail@host2.com', 'rm_tmc' )
            )
        );

        $this->pageFactory->addSettingFields(
            'postTypes',    //  section_id
            array(
                'field_id'              =>  'chosen',
                'type'                  =>  'checkbox',
                'title'                 =>  null,
                'label'                 =>  $this->getAllPostTypesNames(),
                'before_field'          =>  sprintf( '<p style="padding-bottom: 1em;">%1$s</p>', __( 'Plugin will only work on chosen post types.', 'rm_tmc' ) )
            )
        );

        $this->pageFactory->addSettingFields(
            'notifications',    //  section_id
            array(
                'field_id'              =>  'title',
                'type'                  =>  'text',
                'title'                 =>  __( 'Title', 'rm_tmc' ),
                'attributes'            =>  array(
                    'class'                 =>  'regular-text'
                ),
                'tip'                   =>  __( 'You may also use codes like %post_title%', 'rm_tmc' )
            ),
            array(
                'field_id'              =>  'content',
                'type'                  =>  'textarea',
                'title'                 =>  __( 'Content', 'rm_tmc' ),
                'rich'                  =>  array(
                    'tinymce'               =>  array(
                        'wp_autoresize_on'      =>  true,
                        'toolbar1'              =>  'formatselect,bold,italic,underline,forecolor,bullist,numlist,link,unlink',
                        'width'                 =>  500
                    )
                ),
                'before_field'          =>  $this->getDisplayOfTemplatePicker(),
                'after_field'           =>  $this->getDisplayOfTemplateCodes(),
                'attributes'            =>  array()
            ),
            array(
                'field_id'              =>  'emailAddress',
                'type'                  =>  'email',
                'title'                 =>  __( 'Quick test', 'rm_tmc' ),
                'attributes'            =>  array(
                    'class'                 =>  'regular-text'
                ),
                'value'                 =>  wp_get_current_user()->user_email,
                'description'           =>  __( 'Insert your e-mail address and send preview of notification.', 'rm_tmc' ),
                'after_input'           =>  sprintf( '
                                                <button
                                                class="button"
                                                disabled="disabled"
                                                data-quick-test-action="%1$s"
                                                data-target-input-id="#notifications_emailAddress__0"
                                                data-title-input-id="#notifications_title__0"
                                                data-content-tinymce-id="#wp-notifications_content__0-wrap"
                                                data-content-textarea-id="#notifications_content__0"
                                                >%2$s</button>',
                                                Revisions::SEND_QUICK_EMAIL_TEST_ACTION_NAME,
                                                __( 'Send test', 'rm_tmc' )
                                            ),
                'save'                  =>  false
            )
        );

        $this->pageFactory->addSettingFields(
            'submit',    //  section_id
            array(
                'field_id'              =>  'submitForm',
                'type'                  =>  'submit',
                'title'                 =>  null,
                'value'                 =>  __( 'Update settings', 'rm_tmc' )
            )
        );

    }

    //  ================================================================================
    //  FILTERS
    //  ================================================================================

    /**
     * @param array $fields
     *
     * @return array
     */
    public function _f_redefineField_capabilities_excludedEmails( $fields ){

        if( ! App::i()->license->isActive() ){

            $content = sprintf( '<a class="pro-link" href="%1$s" target="_blank">%2$s</a>',
                App::TMC_SHOP_URL,
                __( 'PRO version gives you ability to exclude certain e-mail addresses from notifications.', 'rm_tmc' )
            );

            $fields['capabilities']['excludedEmails']['description']    = null;
            $fields['capabilities']['excludedEmails']['content']        = $content;

        }

        return $fields;

    }

    /**
     * @param array $fields
     *
     * @return array
     */
    public function _f_redefineField_postTypes_chosen( $fields ) {

        if( ! App::i()->license->isActive() ){

            //  Attributes

            $attrsDefinition    = array();
            $postTypeNames      = $this->getAllPostTypesNames();

            foreach( $postTypeNames as $key => $value ){

                //  Disable non-default post type

                if( $key !== 'post' ){
                    $attrsDefinition[ $key ] = array(
                        'disabled'  =>  'disabled'
                    );
                }

            }

            $fields['postTypes']['chosen']['attributes'] = $attrsDefinition;

            //  Info

            $fields['postTypes']['chosen']['after_field'] = sprintf( '<div style="clear: both; padding-top: 1em;"><a class="pro-link" href="%1$s" target="_blank">%2$s</a></div>',
                                                                App::TMC_SHOP_URL,
                                                                __( 'You can choose every custom post type in PRO version.', 'rm_tmc' )
                                                            );

        }

        return $fields;

    }

    /**
     * @param array $fields
     *
     * @return array
     */
    public function _f_redefineField_notifications_content( $fields ) {

        if( ! App::i()->license->isActive() ){

            $previewHtml = sprintf( '<div><div style="display: inline-block;">%1$s</div><p style="padding-top: 1em;"><a class="pro-link" href="%2$s" target="_blank">%3$s</a></p></div>',
                App::i()->settings->getNotificationContent(),
                App::TMC_SHOP_URL,
                __( 'PRO version gives you ability to edit this content.', 'rm_tmc' )
            );

            $fields['notifications']['content']['after_field']                  = $previewHtml;
            $fields['notifications']['content']['attributes']['field']['style'] = 'display: none;';

        }

        return $fields;

    }

    //  ================================================================================
    //  FUNCTIONALITY
    //  ================================================================================

    /**
     * @return string[]
     */
    protected function getAllCapabilitiesNames() {

        $role               = get_role( 'administrator' );    //  Administrator is used for listing all possible capabilities
        $capabilitiesNames  = array();

        if( $role ){
            foreach( $role->capabilities as $key => $value ) {
                $capabilitiesNames[$key] = $key;
            }
        }

        return $capabilitiesNames;

    }

    /**
     * @return string[]
     */
    protected function getAllRolesNames() {

        $roles = wp_roles()->role_objects;
        $names = wp_roles()->role_names;

        $rolesNames         = array();
        $rolesNames['']     = __( '- Do not send -', 'rm_tmc' );

        foreach( $roles as $role ) {    /** @var WP_Role $role */
            $rolesNames[ $role->name ] = array_key_exists( $role->name, $names ) ? $names[ $role->name ] : $role->name;
        }

        return $rolesNames;

    }

    /**
     * @return string[]
     */
    protected function getAllPostTypesNames() {

        /** @var WP_Post_Type[] $postTypesObjects */
        $postTypesObjects = get_post_types( array( 'show_ui' => true ), 'objects' );

        $postTypes      = array();
        $excludeList    = array( 'attachment' );

        foreach( $postTypesObjects as $postType ){
            if( ! in_array( $postType->name, $excludeList ) ){  //  Exclude if in list

                $postTypes[ $postType->name ] = sprintf( '<span  style="display: inline-block; width: 150px; margin-right: 1em;">%1$s</span> <i>( %2$s )</i>',
                    $postType->label,
                    $postType->name
                );

            }
        }

        return $postTypes;

    }

    /**
     * @return string
     */
    protected function getDisplayOfTemplatePicker() {

        //  ----------------------------------------
        //  Prepare data
        //  ----------------------------------------

        $templates = array(
            'default_mail',
            'blue_mail',
            'mono_mail',
            'light_mail',
            'aqua_mail',
            'moonlight_mail'
        );

        $templatesDirUrl = App::s()->getUrl( '/assets/emailTemplates' );

        //  ----------------------------------------
        //  Build display
        //  ----------------------------------------

        $html = '<div class="template_picker">';

        foreach( $templates as $templateName ){

            $html .= '<div class="cell">';

            $templateHtmlObj = new HtmlElement( 'a' );
            $templateHtmlObj->addAttributes( array(
                'href'              =>  App::i()->license->isActive() ? '#' : App::TMC_SHOP_URL,
                'data-template-url' =>  $templatesDirUrl . '/' . $templateName . '.html?ver=' . App::s()->getFullPluginVersion(),
                'class'             =>  App::i()->license->isActive() ? 'template' : 'template disabled',
                'target'            =>  '_blank'
            ) );

            $imgUrl = $templatesDirUrl . '/' . $templateName . '.jpg';

            $html .= $templateHtmlObj->setContent( sprintf( '<img src="%1$s" alt="">', $imgUrl ) )->getDisplay();
            $html .= '</div>';

        }

        $html .= '</div>';

        return $html;

    }

    /**
     * @return string
     */
    public function getDisplayOfTemplateCodes() {

        $html = <<<HTML
        
        <div style="clear: both; padding-top: 1em;">
            <p><b>Use codes below to personalize your e-mail notification:</b></p>

            <ul>
                <li><code>%post_url%</code> - URL of post visualization.</li>
                <li><code>%post_title%</code> - Title of this post.</li>
                <li><code>%post_admin_url%</code> - URL of admin page, where you can accept changes.</li>
                <li><code>%author_name%</code> - Author of this revision.</li>
                <li><code>%author_mail%</code> - His/her e-mail address.</li>
                <li><code>%site_url%</code> - Dynamic link of your website.</li>
            </ul>
        </div>

HTML;

        return $html;

    }

}