<?php
namespace tmc\revisionmanager\src;

/**
 * @author jakubkuranda@gmail.com
 * Date: 06.03.2018
 * Time: 10:21
 */

use shellpress\v1_2_0\ShellPress;
use tmc\revisionmanager\src\Components\DashboardWidget;
use tmc\revisionmanager\src\Components\License;
use tmc\revisionmanager\src\Components\Revisions;
use tmc\revisionmanager\src\Components\Settings;
use tmc\revisionmanager\src\Components\Upgrade;
use tmc\revisionmanager\src\Components\Utilities;
use TMC_revision_manager_AdminPageFramework;

class App extends ShellPress {

    const TMC_SHOP_URL = 'https://themastercut.co/plugins/revision-manager-tmc/?utm_source=client&utm_medium=plugin&utm_campaign=revision-manager-tmc';

    /** @var License */
    public $license;

    /** @var Settings */
    public $settings;

    /** @var Revisions */
    protected $revisions;

    /** @var Utilities */
    public $utilities;

    /** @var Upgrade */
    protected $upgrade;

    /** @var DashboardWidget */
    protected $dashboardWidget;

    /**
     * Called automatically after core is ready.
     *
     * @return void
     */
    protected function onSetUp() {
        //  ----------------------------------------
        //  Namespace autoloading
        //  ----------------------------------------

        App::s()->autoloading->addNamespace( 'tmc\revisionmanager', dirname( App::s()->getMainPluginFile() ) );

        //  ----------------------------------------
        //  Default options
        //  ----------------------------------------

        App::s()->options->setDefaultOptions(
            array(
                'internal'          =>  array(
                    'doneUpgrade'       =>  '0'
                ),
                'capabilities'      =>  array(
                    'capCopy'           =>  'edit_posts',
                    'capAccept'         =>  'publish_posts',
                    'roleNotification'  =>  'administrator',
                    'excludedEmails'    =>  null
                ),
                'postTypes'         =>  array(
                    'chosen'            =>  array(
                        'post'              =>  '1'
                    )
                ),
                'notifications'     =>  array(
                    'title'             =>  __( 'Revision Manager TMC - Accept changes', 'rm_tmc' ),
                    'content'           =>  file_get_contents( App::s()->getPath( '/assets/emailTemplates/default_mail.html' ) )
                ),
                'license'                   =>  array(
                    'key'                       =>  null,
                    'keyExpiryDatetime'         =>  null,
                    'lastCheckDatetime'         =>  null,
                    'keyStatus'                 =>  null,
                    'isKeyCorrect'              =>  '0'
                )
            )
        );

        App::s()->event->addOnActivate( array( $this, '_a_defaultOptionsOnActivate' ) );

        //  Support update compatibility ( when plugin is not activated but upgraded )

        if( ! App::s()->options->get() ){
            $this->_a_defaultOptionsOnActivate();
        }

        //  ----------------------------------------
        //  Components
        //  ----------------------------------------

        $this->upgrade          = new Upgrade();
        $this->license          = new License( $this );
        $this->settings         = new Settings();
        $this->utilities        = new Utilities();
        $this->revisions        = new Revisions();
        $this->dashboardWidget  = new DashboardWidget();

        //  ----------------------------------------
        //  Actions
        //  ----------------------------------------

        add_action( 'admin_enqueue_scripts',                array( $this, '_a_enqueueAdminStyles' ) );

        //  ----------------------------------------
        //  Admin pages
        //  ----------------------------------------

        if( is_admin() && ! wp_doing_ajax() && ! defined( 'DOING_CRON' ) ){   //  Keep it lightweight

            App::s()->requireFile( 'lib/tmc-admin-page-framework/admin-page-framework.php', 'TMC_v1_0_3_AdminPageFramework' );
            App::s()->requireFile( 'src/AdminPages/TMC_revision_manager_AdminPageFramework.php' );

            new TMC_revision_manager_AdminPageFramework(
                App::s()->options->getOptionsKey(),     //  Options
                App::s()->getMainPluginFile(),          //  Main file
                'manage_options',                           //  Capability
                App::s()->getPrefix()                   //  Text domain
            );

        }

    }

    //  ================================================================================
    //  ACTIONS
    //  ================================================================================

    /**
     * Called on app activation.
     * Fills differences between default and saved options.
     *
     * @return void
     */
    public function _a_defaultOptionsOnActivate() {

        App::s()->options->fillDifferencies();
        App::s()->options->flush();

    }

    /**
     * @return void
     */
    public function _a_enqueueAdminStyles() {

        wp_enqueue_style( 'rm_tmc_shared', App::s()->getUrl( '/assets/css/shared.css' ), array(), App::s()->getFullPluginVersion() );

    }

}