<?php
namespace tmc\revisionmanager\src\Components;

/**
 * @author jakubkuranda@gmail.com
 * Date: 14.03.2018
 * Time: 10:38
 */

use tmc\revisionmanager\src\App;

class Upgrade {

    /**
     * Upgrade constructor.
     */
    public function __construct() {

        //  ----------------------------------------
        //  Do upgrade on normal run
        //  ----------------------------------------

        if( ! App::s()->options->get( 'internal/doneUpgrade' ) ){
            $this->doUpgrade();
        }

        //  ----------------------------------------
        //  Actions
        //  ----------------------------------------

        App::s()->event->addOnActivate( array( $this, '_a_doUpgradeOnActivate' ) );

    }

    /**
     * Performs upgrade from old options to new.
     * When done, removes old options from database.
     * It flushes new options.
     *
     * @return void
     */
    protected function doUpgrade() {

        $oldOptionKeys = array(
            'rm_tmc_post_types'         =>  'postTypes/chosen',
            'rm_tmc_editor_cap'         =>  'capabilities/capCopy',
            'rm_tmc_accept_cap'         =>  'capabilities/capAccept',
            'rm_tmc_mail_role'          =>  'capabilities/roleNotification',
            'rm_tmc_emails_exclude'     =>  'capabilities/excludedEmails',
            'rm_tmc_mail_content'       =>  'notifications/content',
            'rm_tmc_mail_title'         =>  'notifications/title',
            'rm_tmc_options_version'    =>  null
        );

        foreach( $oldOptionKeys as $oldOptionKey => $newOptionKey ){

            $oldOption = get_option( $oldOptionKey, null );

            //  Perform upgrade

            if( $oldOption && ! empty( $newOptionKey ) ){

                if( $oldOptionKey === 'rm_tmc_post_types' ){    //  Only this option has custom structure.

                    App::s()->options->set( $newOptionKey, $this->parseOldChosenPostTypesOptionToNew( (array) $oldOption ) );

                } else {

                    App::s()->options->set( $newOptionKey, $oldOption );

                }

            }

            //  Remove old option

            delete_option( $oldOptionKey );

            App::s()->log->info( sprintf( 'UPGRADE: %1$s => %2$s', $oldOptionKey, $newOptionKey ) );

        }

        App::s()->options->set( 'internal/doneUpgrade', '1' );
        App::s()->options->flush();

    }

    /**
     * Transforms old chosen post types option to new structure.
     *
     * @param array $oldOption
     *
     * return array
     */
    protected function parseOldChosenPostTypesOptionToNew( $oldOption ) {

        $newOption = array();

        foreach( (array) $oldOption as $postType => $value ){

            if(
                is_array( $value )
                && array_key_exists( 'checked', $value )
                && $value['checked'] == '1'
            ){
                $newOption[ $postType ] = '1';
            } else {
                $newOption[ $postType ] = '0';
            }

        }

        return $newOption;

    }

    //  ================================================================================
    //  ACTIONS
    //  ================================================================================

    /**
     * Called on plugin activation.
     *
     * @return void
     */
    public function _a_doUpgradeOnActivate() {

        $this->doUpgrade();

    }

}